import osmnx as ox
import networkx as nx
import folium
import itertools
import pickle
import time, datetime
import os.path


ox.settings.log_console=True
ox.settings.use_cache=True

print('OSMnx version: ', ox.__version__)
print('NetworkX version: ',nx.__version__)

start_time = time.time()

def elapsed_time(start):
    end_time = time.time()
    duration = round(end_time - start)
    timestamp = str(datetime.timedelta(seconds = duration))
    print(timestamp)

# read graph
place     = 'Gila National Forest, New Mexico, United States'
# Using network_type='walk' seems to include dirt roads, so use a custom filter
cf = '["highway"~"path|footway"]'
# create graph from OSM within the boundaries of some 
# geocodable place(s)
G = ox.graph_from_place(place,simplify=False,retain_all=True,custom_filter=cf)


G = G.to_undirected() # we prefer to work with an undirected multigraph


  
def report(Q, txt=''):
    print(len(Q.nodes()), ' nodes ', txt)
    print(len(Q.edges()), ' edges ', txt)
    if isinstance(Q,nx.MultiGraph):    
        length = ox.stats.edge_length_total(Q) #in meters
        length = length * 0.0006213712
        print(f'total length {length:.2f} miles ', txt)
    zeroes = 0    
    odds = 0
    evens = 0
    twos = 0
    ones = 0
    for node in Q.nodes:
        d = Q.degree(node)
        if d == 0:
            zeroes += 1
        if d % 2 == 1:
            odds += 1
        else:
            evens += 1
        if d == 2:
            twos += 1
        if d == 1:
            ones += 1
    if zeroes > 0:            
        print('number of zero-degree nodes ' + txt + ' :', zeroes)            
    print('number of odd intersections ' + txt + ' :', odds)
    print('number of even intersections ' + txt + ' :', evens)
    print('number of 2-way intersections ' + txt + ' :', twos)
    print('number of trailhead/terminus ' + txt + ' :', ones)
    print('')
        
        

# get undirected multigraph
H = ox.utils_graph.get_undirected(G)
elapsed_time(start_time)
report(H,'in undirected multigraph')

pickle.dump(H,open('h_graph.pkl', 'wb'))
print('finished script ' + os.path.basename(__file__))






